// Default source code for Mavigator's Active Extensions

using System.Windows.Forms;
using System.Collections;
using com.mt.AbstractDatabase2;
using com.mt.Mavigator.ActiveExtensions;
using System;

namespace com.mt.Mavigator.ActiveExtensions {
	public class ActiveExtensionInstance { 
		ActiveExtensionInstance() {} 

		/// <summary>
		/// Shows number of objects in the given list.
		/// </summary>
		/// <param name="IDs"></param>
		/// <param name="wrapper"></param>
		public static void ShowNumberOfMarkedObjects(IList IDs, AdWrapper wrapper) {
			if(IDs == null) {
				MessageBox.Show("There are no marked objects defined for this class!", "Wrapper version " + wrapper.Version);
			}
			else {
				MessageBox.Show("Number of marked objects: " + IDs.Count, "Wrapper version " + wrapper.Version);
			}
		}

		/// <summary>
		/// Returns min value of the given attribute for a given set of objects.
		/// </summary>
		/// <param name="IDs"></param>
		/// <param name="wrapper"></param>
		public static void FindMinimumAttributeValue(IList IDs, AdWrapper wrapper) {
			int counter = 0;
			AdObject minObject = null;

			// Check if list is not empty
			if(IDs == null || IDs.Count < 1) {
				MessageBox.Show("There are no objects to process!");
				return;
			}

			// Get id of the first object
			AdUniqueID id1 = (AdUniqueID) IDs[0];

			// Get class's id for first object
			AdUniqueID idClass = wrapper.GetClassID(id1);

			// Show form to select an attribute
			SelectAttributeForm form = new SelectAttributeForm(wrapper, idClass);
			form.ShowDialog();

			if(form.DialogResult == DialogResult.Cancel || form.SelectedAttribute == null) {
				// Cancelled
				return;
			}

			if(!form.SelectedAttribute.IsNumber) {
				MessageBox.Show("Selected attribute is not a number!", "Warning");
				return;
			}

			AdAttribute selectedAttribute = form.SelectedAttribute;
			form.Dispose();

			// Find min value
			double min = ActiveExtensionsUsefulMethods.GetMinValue(wrapper, IDs, selectedAttribute, ref minObject, ref counter);

			// Show the result
			String result = "Minimum value is: " + min.ToString() + "\n"
				+ "for object: " + minObject.Label;
			if(counter != IDs.Count) {
				result += "\nNot all objects (" + counter.ToString() + "/" + IDs.Count.ToString() + ") have been processed.";
			}
			MessageBox.Show(result, "Minimum value");
		}	

		/// <summary>
		/// Gets an avearage value for selected attribute among objects (defined by its ids).
		/// </summary>
		/// <param name="IDs"></param>
		/// <param name="wrapper"></param>
		public static void FindAverageAttributeValue(IList IDs, AdWrapper wrapper) {
			double sum = 0;
			int counter = 0;

			// Check if list is not empty
			if(IDs == null || IDs.Count < 1) {
				MessageBox.Show("There are no objects to process!");
				return;
			}

			// Get id of the first object
			AdUniqueID id1 = (AdUniqueID) IDs[0];

			// Get class's id for first object
			AdUniqueID idClass = wrapper.GetClassID(id1);

			// Show form to select an attribute
			SelectAttributeForm form = new SelectAttributeForm(wrapper, idClass);
			form.ShowDialog();

			if(form.DialogResult == DialogResult.Cancel || form.SelectedAttribute == null) {
				// Cancelled
				return;
			}

			if(!form.SelectedAttribute.IsNumber) {
				MessageBox.Show("Selected attribute is not a number!", "Warning");
				return;
			}

			AdAttribute selectedAttribute = form.SelectedAttribute;
			form.Dispose();

			// Process objects
			foreach(AdUniqueID id in IDs) {
				AdObject obj = wrapper.GetObject(id);

				// Find value of the attribute
				foreach(AdAttributeValue v in obj.Values) {
					if(v.MetaAttributeID == selectedAttribute.UniqueID) {
						// This is the attribute
						counter++;
						double curVal = Convert.ToDouble(v.Value);
						sum += curVal;
						break;
					}
				}
			}

			// Show the result
			double avg = sum / counter;
			String result = "Average value is: " + avg.ToString();
			if(counter != IDs.Count) {
				result += "\nNot all objects (" + counter.ToString() + "/" + IDs.Count.ToString() + ") have been processed.";
			}
			MessageBox.Show(result, "Minimum value");
		}	

		public static void ActiveProjection(IList IDs, AdWrapper wrapper) {
			int counter = 0;
			AdObject minObject = null;
			SelectAttributeForm form = null;

			// Check if list is not empty
			if(IDs == null || IDs.Count < 1) {
				MessageBox.Show("There are no objects to process!");
				return;
			}

			// Get id of the first object
			AdUniqueID id1 = (AdUniqueID) IDs[0];

			// Get class's id for first object
			AdUniqueID idClass = wrapper.GetClassID(id1);

			// Show form to select an attribute for X-axis
			form = new SelectAttributeForm(wrapper, idClass);
			form.ShowDialog();
			if(form.DialogResult == DialogResult.Cancel || form.SelectedAttribute == null) {
				// Cancelled
				return;
			}
			if(!form.SelectedAttribute.IsNumber) {
				MessageBox.Show("Selected attribute is not a number!", "Warning");
				return;
			}
			AdAttribute selectedAttributeX = form.SelectedAttribute;
			form.Dispose();

			// Show form to select an attribute for Y-axis
			form = new SelectAttributeForm(wrapper, idClass);
			form.ShowDialog();
			if(form.DialogResult == DialogResult.Cancel || form.SelectedAttribute == null) {
				// Cancelled
				return;
			}
			if(!form.SelectedAttribute.IsNumber) {
				MessageBox.Show("Selected attribute is not a number!", "Warning");
				return;
			}
			AdAttribute selectedAttributeY = form.SelectedAttribute;
			form.Dispose();

			ActiveProjectionForm formAP = new ActiveProjectionForm(wrapper, selectedAttributeX, selectedAttributeY, IDs);
			formAP.Show();
		}
	}
}
